package java.nio;

import java.util.Spliterator;
import java.util.stream.StreamSupport;
import java.util.stream.IntStream;
import java.io.IOException;

public abstract class CharBuffer extends Buffer implements Comparable<CharBuffer>, Appendable, CharSequence, Readable
{
    final char[] hb;
    final int offset;
    boolean isReadOnly;
    
    CharBuffer(int n, int n2, int n3, int n4, char[] hb, int offset) {
        super(n, n2, n3, n4);
        this.hb = hb;
        this.offset = offset;
    }
    
    CharBuffer(int n, int n2, int n3, int n4) {
        this(n, n2, n3, n4, null, 0);
    }
    
    public static CharBuffer allocate(int n) {
        if (n < 0) {
            throw new IllegalArgumentException();
        }
        return new HeapCharBuffer(n, n);
    }
    
    public static CharBuffer wrap(char[] array, int n, int n2) {
        try {
            return new HeapCharBuffer(array, n, n2);
        }
        catch (IllegalArgumentException ex) {
            throw new IndexOutOfBoundsException();
        }
    }
    
    public static CharBuffer wrap(char[] array) {
        return wrap(array, 0, array.length);
    }
    
    @Override
    public int read(CharBuffer charBuffer) throws IOException {
        int remaining = charBuffer.remaining();
        int remaining2 = this.remaining();
        if (remaining2 == 0) {
            return -1;
        }
        int min = Math.min(remaining2, remaining);
        int limit = this.limit();
        if (remaining < remaining2) {
            this.limit(this.position() + min);
        }
        try {
            if (min > 0) {
                charBuffer.put(this);
            }
        }
        finally {
            this.limit(limit);
        }
        return min;
    }
    
    public static CharBuffer wrap(CharSequence charSequence, int n, int n2) {
        try {
            return new StringCharBuffer(charSequence, n, n2);
        }
        catch (IllegalArgumentException ex) {
            throw new IndexOutOfBoundsException();
        }
    }
    
    public static CharBuffer wrap(CharSequence charSequence) {
        return wrap(charSequence, 0, charSequence.length());
    }
    
    public abstract CharBuffer slice();
    
    public abstract CharBuffer duplicate();
    
    public abstract CharBuffer asReadOnlyBuffer();
    
    public abstract char get();
    
    public abstract CharBuffer put(char p0);
    
    public abstract char get(int p0);
    
    abstract char getUnchecked(int p0);
    
    public abstract CharBuffer put(int p0, char p1);
    
    public CharBuffer get(char[] array, int i, int n) {
        Buffer.checkBounds(i, n, array.length);
        if (n > this.remaining()) {
            throw new BufferUnderflowException();
        }
        int n2;
        for (n2 = i + n, ; i < n2; ++i) {
            array[i] = this.get();
        }
        return this;
    }
    
    public CharBuffer get(char[] array) {
        return this.get(array, 0, array.length);
    }
    
    public CharBuffer put(CharBuffer charBuffer) {
        if (charBuffer == this) {
            throw new IllegalArgumentException();
        }
        if (this.isReadOnly()) {
            throw new ReadOnlyBufferException();
        }
        int remaining = charBuffer.remaining();
        if (remaining > this.remaining()) {
            throw new BufferOverflowException();
        }
        for (int i = 0; i < remaining; ++i) {
            this.put(charBuffer.get());
        }
        return this;
    }
    
    public CharBuffer put(char[] array, int i, int n) {
        Buffer.checkBounds(i, n, array.length);
        if (n > this.remaining()) {
            throw new BufferOverflowException();
        }
        int n2;
        for (n2 = i + n, ; i < n2; ++i) {
            this.put(array[i]);
        }
        return this;
    }
    
    public final CharBuffer put(char[] array) {
        return this.put(array, 0, array.length);
    }
    
    public CharBuffer put(String s, int i, int n) {
        Buffer.checkBounds(i, n - i, s.length());
        if (this.isReadOnly()) {
            throw new ReadOnlyBufferException();
        }
        if (n - i > this.remaining()) {
            throw new BufferOverflowException();
        }
        for (; i < n; ++i) {
            this.put(s.charAt(i));
        }
        return this;
    }
    
    public final CharBuffer put(String s) {
        return this.put(s, 0, s.length());
    }
    
    @Override
    public final boolean hasArray() {
        return this.hb != null && !this.isReadOnly;
    }
    
    @Override
    public final char[] array() {
        if (this.hb == null) {
            throw new UnsupportedOperationException();
        }
        if (this.isReadOnly) {
            throw new ReadOnlyBufferException();
        }
        return this.hb;
    }
    
    @Override
    public final int arrayOffset() {
        if (this.hb == null) {
            throw new UnsupportedOperationException();
        }
        if (this.isReadOnly) {
            throw new ReadOnlyBufferException();
        }
        return this.offset;
    }
    
    public abstract CharBuffer compact();
    
    @Override
    public abstract boolean isDirect();
    
    @Override
    public int hashCode() {
        int n = 1;
        for (int position = this.position(), i = this.limit() - 1; i >= position; --i) {
            n = 31 * n + this.get(i);
        }
        return n;
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof CharBuffer)) {
            return false;
        }
        CharBuffer charBuffer = (CharBuffer)o;
        if (this.remaining() != charBuffer.remaining()) {
            return false;
        }
        int position = this.position();
        int i = this.limit() - 1;
        int n = charBuffer.limit() - 1;
        while (i >= position) {
            if (!equals(this.get(i), charBuffer.get(n))) {
                return false;
            }
            --i;
            --n;
        }
        return true;
    }
    
    private static boolean equals(char c, char c2) {
        return c == c2;
    }
    
    @Override
    public int compareTo(CharBuffer charBuffer) {
        int n = this.position() + Math.min(this.remaining(), charBuffer.remaining());
        int i = this.position();
        int position = charBuffer.position();
        while (i < n) {
            int compare = compare(this.get(i), charBuffer.get(position));
            if (compare != 0) {
                return compare;
            }
            ++i;
            ++position;
        }
        return this.remaining() - charBuffer.remaining();
    }
    
    private static int compare(char x, char y) {
        return Character.compare(x, y);
    }
    
    @Override
    public String toString() {
        return this.toString(this.position(), this.limit());
    }
    
    abstract String toString(int p0, int p1);
    
    @Override
    public final int length() {
        return this.remaining();
    }
    
    @Override
    public final char charAt(int n) {
        return this.get(this.position() + this.checkIndex(n, 1));
    }
    
    @Override
    public abstract CharBuffer subSequence(int p0, int p1);
    
    @Override
    public CharBuffer append(CharSequence charSequence) {
        if (charSequence == null) {
            return this.put("null");
        }
        return this.put(charSequence.toString());
    }
    
    @Override
    public CharBuffer append(CharSequence charSequence, int n, int n2) {
        charSequence = ((charSequence == null) ? "null" : charSequence);
        return this.put(charSequence.subSequence(n, n2).toString());
    }
    
    @Override
    public CharBuffer append(char c) {
        return this.put(c);
    }
    
    public abstract ByteOrder order();
    
    @Override
    public IntStream chars() {
        return StreamSupport.intStream(()/*  */ -> new CharBufferSpliterator(this), 16464, false);
    }
}
